#![feature(round_char_boundary)]
use std::path::Path;

#[cfg(feature = "compare")]
use ::organic::parser::document;
#[cfg(feature = "compare")]
use organic::compare_document;
#[cfg(feature = "compare")]
use organic::emacs_parse_org_document;
#[cfg(feature = "compare")]
use organic::parser::sexp::sexp_with_padding;

#[cfg(feature = "tracing")]
use crate::init_tracing::init_telemetry;
#[cfg(feature = "tracing")]
use crate::init_tracing::shutdown_telemetry;
#[cfg(feature = "tracing")]
mod init_tracing;

#[cfg(not(feature = "tracing"))]
fn main() -> Result<(), Box<dyn std::error::Error>> {
    main_body()
}

#[cfg(feature = "tracing")]
fn main() -> Result<(), Box<dyn std::error::Error>> {
    let rt = tokio::runtime::Runtime::new()?;
    let result = rt.block_on(async {
        init_telemetry()?;
        let main_body_result = main_body();
        shutdown_telemetry()?;
        main_body_result
    });
    result
}

#[cfg_attr(feature = "tracing", tracing::instrument(ret, level = "debug"))]
fn main_body() -> Result<(), Box<dyn std::error::Error>> {
    run_compare(
        std::env::args()
            .nth(1)
            .ok_or("Pass a single file into this script.")?,
    )
}

#[cfg(feature = "compare")]
fn run_compare<P: AsRef<Path>>(todo_org_path: P) -> Result<(), Box<dyn std::error::Error>> {
    let org_contents = std::fs::read_to_string(todo_org_path.as_ref())?;
    let (remaining, rust_parsed) = document(org_contents.as_str()).expect("Org Parse failure");
    let org_sexp = emacs_parse_org_document(todo_org_path.as_ref())?;
    let (_remaining, parsed_sexp) =
        sexp_with_padding(org_sexp.as_str()).expect("Sexp Parse failure");

    println!("{}\n\n\n", org_contents.as_str());
    println!("{}", org_sexp);
    println!("{:#?}", rust_parsed);

    // We do the diffing after printing out both parsed forms in case the diffing panics
    let diff_result = compare_document(&parsed_sexp, &rust_parsed)?;
    diff_result.print()?;

    if diff_result.is_bad() {
        Err("Diff results do not match.")?;
    }
    if remaining != "" {
        Err(format!("There was unparsed text remaining: {}", remaining))?;
    }

    Ok(())
}

#[cfg(not(feature = "compare"))]
fn run_compare<P: AsRef<Path>>(_todo_org_path: P) -> Result<(), Box<dyn std::error::Error>> {
    println!("This program was built with compare disabled. Doing nothing.");
    Ok(())
}
